@@ -16,15 +16,11 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder;
 
 import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
-import org.gradle.api.attributes.Attribute;
-import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ComponentSelectorConverter;
 import org.gradle.api.internal.artifacts.ResolvedVersionConstraint;
 import org.gradle.api.internal.artifacts.configurations.ConflictResolution;
@@ -44,17 +40,18 @@ import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflict
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.ModuleConflictHandler;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.PotentialConflict;
 import org.gradle.api.internal.attributes.AttributeDesugaring;
-import org.gradle.api.internal.attributes.AttributesSchemaInternal;
-import org.gradle.api.internal.attributes.CompatibilityRule;
-import org.gradle.api.internal.attributes.ImmutableAttributes;
+import org.gradle.api.internal.attributes.AttributeSchemaServices;
 import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
+import org.gradle.api.internal.attributes.immutable.ImmutableAttributesSchema;
+import org.gradle.api.internal.attributes.matching.AttributeMatcher;
+import org.gradle.api.internal.capabilities.CapabilityInternal;
 import org.gradle.api.specs.Spec;
-import org.gradle.internal.component.ResolutionFailureHandler;
 import org.gradle.internal.component.model.ComponentGraphResolveMetadata;
 import org.gradle.internal.component.model.ComponentIdGenerator;
-import org.gradle.internal.component.model.DefaultCompatibilityCheckResult;
 import org.gradle.internal.component.model.DependencyMetadata;
 import org.gradle.internal.component.model.GraphVariantSelector;
+import org.gradle.internal.component.model.VariantGraphResolveMetadata;
+import org.gradle.internal.component.resolution.failure.ResolutionFailureHandler;
 import org.gradle.internal.component.resolution.failure.exception.AbstractResolutionFailureException;
 import org.gradle.internal.deprecation.DeprecationLogger;
 import org.gradle.internal.operations.BuildOperationConstraint;
@@ -65,7 +62,6 @@ import org.gradle.internal.resolve.resolver.DependencyToComponentIdResolver;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import javax.annotation.Nullable;
 import javax.inject.Inject;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -74,8 +70,8 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.Objects;
 import java.util.Set;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.stream.Collectors;
 
 public class DependencyGraphBuilder {
@@ -87,6 +83,7 @@ public class DependencyGraphBuilder {
 
     private final ModuleExclusions moduleExclusions;
     private final ImmutableAttributesFactory attributesFactory;
+    private final AttributeSchemaServices attributeSchemaServices;
     private final AttributeDesugaring attributeDesugaring;
     private final VersionSelectorScheme versionSelectorScheme;
     private final VersionComparator versionComparator;
@@ -99,6 +96,7 @@ public class DependencyGraphBuilder {
     public DependencyGraphBuilder(
         ModuleExclusions moduleExclusions,
         ImmutableAttributesFactory attributesFactory,
+        AttributeSchemaServices attributeSchemaServices,
         AttributeDesugaring attributeDesugaring,
         VersionSelectorScheme versionSelectorScheme,
         VersionComparator versionComparator,
@@ -109,6 +107,7 @@ public class DependencyGraphBuilder {
     ) {
         this.moduleExclusions = moduleExclusions;
         this.attributesFactory = attributesFactory;
+        this.attributeSchemaServices = attributeSchemaServices;
         this.attributeDesugaring = attributeDesugaring;
         this.versionSelectorScheme = versionSelectorScheme;
         this.versionComparator = versionComparator;
@@ -122,7 +121,6 @@ public class DependencyGraphBuilder {
         RootComponentMetadataBuilder.RootComponentState rootComponent,
         List<? extends DependencyMetadata> syntheticDependencies,
         Spec<? super DependencyMetadata> edgeFilter,
-        AttributesSchemaInternal consumerSchema,
         ComponentSelectorConverter componentSelectorConverter,
         DependencyToComponentIdResolver componentIdResolver,
         ComponentMetaDataResolver componentMetaDataResolver,
@@ -135,10 +133,8 @@ public class DependencyGraphBuilder {
         boolean failingOnChangingVersions,
         DependencyGraphVisitor modelVisitor
     ) {
-        ResolutionConflictTracker conflictTracker = new ResolutionConflictTracker(
-            new DefaultConflictHandler(moduleConflictResolver, moduleReplacements),
-            new DefaultCapabilitiesConflictHandler(capabilityConflictResolvers)
-        );
+        ModuleConflictHandler moduleConflictHandler = new DefaultConflictHandler(moduleConflictResolver, moduleReplacements);
+        CapabilitiesConflictHandler capabilitiesConflictHandler = new DefaultCapabilitiesConflictHandler(capabilityConflictResolvers);
 
         ResolveState resolveState = new ResolveState(
             idGenerator,
@@ -146,10 +142,10 @@ public class DependencyGraphBuilder {
             componentIdResolver,
             componentMetaDataResolver,
             edgeFilter,
-            consumerSchema,
             moduleExclusions,
             componentSelectorConverter,
             attributesFactory,
+            attributeSchemaServices,
             attributeDesugaring,
             dependencySubstitutionApplicator,
             versionSelectorScheme,
@@ -157,7 +153,8 @@ public class DependencyGraphBuilder {
             versionParser,
             conflictResolution,
             syntheticDependencies,
-            conflictTracker,
+            moduleConflictHandler,
+            capabilitiesConflictHandler,
             variantSelector
         );
 
@@ -175,16 +172,27 @@ public class DependencyGraphBuilder {
         resolveState.onMoreSelected(resolveState.getRoot());
         final List<EdgeState> dependencies = new ArrayList<>();
 
-        ModuleConflictHandler moduleConflictHandler = resolveState.getConflictTracker().getModuleConflictHandler();
-        CapabilitiesConflictHandler capabilitiesConflictHandler = resolveState.getConflictTracker().getCapabilitiesConflictHandler();
+        ModuleConflictHandler moduleConflictHandler = resolveState.getModuleConflictHandler();
+        CapabilitiesConflictHandler capabilitiesConflictHandler = resolveState.getCapabilitiesConflictHandler();
 
         while (resolveState.peek() != null || moduleConflictHandler.hasConflicts() || capabilitiesConflictHandler.hasConflicts()) {
             if (resolveState.peek() != null) {
                 final NodeState node = resolveState.pop();
                 LOGGER.debug("Visiting configuration {}.", node);
 
+                // TODO: Why is this not node.isSelected()?
+                // It seems that node.isSelected can return true while component.isSelected() returns false
+                if (!node.getComponent().isSelected()) {
+                    node.cleanupConstraints();
+                    continue;
+                }
+
                 // Register capabilities for this node
-                registerCapabilities(resolveState, node);
+                if (registerCapabilities(resolveState, node)) {
+                    // We have a conflict, so we need to resolve it first, since this node may not win the conflict.
+                    // There is no reason to continue processing this node otherwise.
+                    continue;
+                }
 
                 // Initialize and collect any new outgoing edges of this node
                 dependencies.clear();
@@ -204,11 +212,19 @@ public class DependencyGraphBuilder {
         }
     }
 
-    private static void registerCapabilities(final ResolveState resolveState, final NodeState node) {
-        CapabilitiesConflictHandler capabilitiesConflictHandler = resolveState.getConflictTracker().getCapabilitiesConflictHandler();
-        node.forEachCapability(capabilitiesConflictHandler, new Action<Capability>() {
+    /**
+     * Detect and register capability conflicts for the given node, deselecting the node if
+     * it is involved in a conflict.
+     *
+     * @return true iff a conflict was detected.
+     */
+    private static boolean registerCapabilities(final ResolveState resolveState, final NodeState node) {
+        AtomicBoolean foundConflict = new AtomicBoolean(false);
+        CapabilitiesConflictHandler capabilitiesConflictHandler = resolveState.getCapabilitiesConflictHandler();
+
+        node.forEachCapability(capabilitiesConflictHandler, new Action<CapabilityInternal>() {
             @Override
-            public void execute(Capability capability) {
+            public void execute(CapabilityInternal capability) {
                 // This is a performance optimization. Most modules do not declare capabilities. So, instead of systematically registering
                 // an implicit capability for each module that we see, we only consider modules which _declare_ capabilities. If they do,
                 // then we try to find a module which provides the same capability. It that module has been found, then we register it.
@@ -217,7 +233,7 @@ public class DependencyGraphBuilder {
                 for (ModuleResolveState state : resolveState.getModules()) {
                     if (state.getId().getGroup().equals(capability.getGroup()) && state.getId().getName().equals(capability.getName())) {
                         Collection<ComponentState> versions = state.getVersions();
-                        implicitProvidersForCapability = Lists.newArrayListWithExpectedSize(versions.size());
+                        implicitProvidersForCapability = new ArrayList<>(versions.size());
                         for (ComponentState version : versions) {
                             List<NodeState> nodes = version.getNodes();
                             for (NodeState nodeState : nodes) {
@@ -235,6 +251,7 @@ public class DependencyGraphBuilder {
                 );
                 if (c.conflictExists()) {
                     c.withParticipatingModules(resolveState.getDeselectVersionAction());
+                    foundConflict.set(true);
                 }
             }
 
@@ -242,21 +259,22 @@ public class DependencyGraphBuilder {
                 return nodeState.getMetadata().getCapabilities().asSet().isEmpty();
             }
         });
+        return foundConflict.get();
     }
 
     private boolean resolveEdges(
         final NodeState node,
         final List<EdgeState> dependencies,
-        final Spec<EdgeState> dependencyFilter,
+        final Spec<EdgeState> edgeFilter,
         final boolean recomputeSelectors,
         final ResolveState resolveState
     ) {
         if (dependencies.isEmpty()) {
             return false;
         }
-        if (performSelectionSerially(dependencies, dependencyFilter, resolveState, recomputeSelectors)) {
-            maybeDownloadMetadataInParallel(node, dependencies, dependencyFilter, buildOperationExecutor, resolveState.getComponentMetadataResolver());
-            attachToTargetRevisionsSerially(dependencies, dependencyFilter);
+        if (performSelectionSerially(dependencies, edgeFilter, resolveState, recomputeSelectors)) {
+            maybeDownloadMetadataInParallel(node, dependencies, edgeFilter, buildOperationExecutor, resolveState.getComponentMetadataResolver());
+            attachToTargetRevisionsSerially(dependencies, edgeFilter);
             return true;
         } else {
             return false;
@@ -264,25 +282,25 @@ public class DependencyGraphBuilder {
 
     }
 
-    private static boolean performSelectionSerially(List<EdgeState> dependencies, Spec<EdgeState> dependencyFilter, ResolveState resolveState, boolean recomputeSelectors) {
+    private static boolean performSelectionSerially(List<EdgeState> edges, Spec<EdgeState> edgeFilter, ResolveState resolveState, boolean recomputeSelectors) {
         boolean processed = false;
-        for (EdgeState dependency : dependencies) {
-            if (!dependencyFilter.isSatisfiedBy(dependency)) {
+        for (EdgeState edge : edges) {
+            if (!edgeFilter.isSatisfiedBy(edge)) {
                 continue;
             }
             if (recomputeSelectors) {
-                dependency.computeSelector();
+                edge.computeSelector();
             }
-            SelectorState selector = dependency.getSelector();
+            SelectorState selector = edge.getSelector();
             ModuleResolveState module = selector.getTargetModule();
 
-            if (selector.canResolve() && module.getSelectors().size() > 0) {
+            if (selector.canAffectSelection() && module.getSelectors().size() > 0) {
                 // Have an unprocessed/new selector for this module. Need to re-select the target version (if there are any selectors that can be used).
                 performSelection(resolveState, module);
             }
-            if (dependency.isUsed()) {
+            if (edge.isUsed()) {
                 // Some corner case result in the edge being removed, in that case it needs to be "removed"
-                module.addUnattachedDependency(dependency);
+                module.addUnattachedEdge(edge);
             }
             processed = true;
         }
@@ -299,7 +317,7 @@ public class DependencyGraphBuilder {
         ComponentState currentSelection = module.getSelected();
 
         try {
-            module.maybeUpdateSelection(resolveState.getConflictTracker());
+            module.maybeUpdateSelection();
         } catch (ModuleVersionResolveException e) {
             // Ignore: All selectors failed, and will have failures recorded
             return;
@@ -314,7 +332,7 @@ public class DependencyGraphBuilder {
 
     private static void checkForModuleConflicts(ResolveState resolveState, ModuleResolveState module) {
         // A new module. Check for conflict with capabilities and module replacements.
-        PotentialConflict c = resolveState.getConflictTracker().getModuleConflictHandler().registerCandidate(module);
+        PotentialConflict c = resolveState.getModuleConflictHandler().registerCandidate(module);
         if (c.conflictExists()) {
             // We have a conflict
             LOGGER.debug("Found new conflicting module {}", module);
@@ -329,13 +347,13 @@ public class DependencyGraphBuilder {
      * Prepares the resolution of edges, either serially or concurrently.
      * It uses a simple heuristic to determine if we should perform concurrent resolution, based on the number of edges, and whether they have unresolved metadata.
      */
-    private static void maybeDownloadMetadataInParallel(NodeState node, List<EdgeState> dependencies, Spec<EdgeState> dependencyFilter, BuildOperationExecutor buildOperationExecutor, ComponentMetaDataResolver componentMetaDataResolver) {
+    private static void maybeDownloadMetadataInParallel(NodeState node, List<EdgeState> edges, Spec<EdgeState> edgeFilter, BuildOperationExecutor buildOperationExecutor, ComponentMetaDataResolver componentMetaDataResolver) {
         List<ComponentState> requiringDownload = null;
-        for (EdgeState dependency : dependencies) {
-            if (!dependencyFilter.isSatisfiedBy(dependency)) {
+        for (EdgeState edge : edges) {
+            if (!edgeFilter.isSatisfiedBy(edge)) {
                 continue;
             }
-            ComponentState targetComponent = dependency.getTargetComponent();
+            ComponentState targetComponent = edge.getTargetComponent();
             if (targetComponent != null && targetComponent.isSelected() && !targetComponent.alreadyResolved()) {
                 if (!componentMetaDataResolver.isFetchingMetadataCheap(targetComponent.getComponentId())) {
                     // Avoid initializing the list if there are no components requiring download (a common case)
@@ -358,34 +376,34 @@ public class DependencyGraphBuilder {
         }
     }
 
-    private static void attachToTargetRevisionsSerially(List<EdgeState> dependencies, Spec<EdgeState> dependencyFilter) {
+    private static void attachToTargetRevisionsSerially(List<EdgeState> edges, Spec<EdgeState> edgeFilter) {
         // the following only needs to be done serially to preserve ordering of dependencies in the graph: we have visited the edges
         // but we still didn't add the result to the queue. Doing it from resolve threads would result in non-reproducible graphs, where
         // edges could be added in different order. To avoid this, the addition of new edges is done serially.
-        for (EdgeState dependency : dependencies) {
-            if (dependencyFilter.isSatisfiedBy(dependency)) {
-                dependency.attachToTargetConfigurations();
+        for (EdgeState edge : edges) {
+            if (edgeFilter.isSatisfiedBy(edge)) {
+                edge.attachToTargetConfigurations();
             }
         }
     }
 
     private static void validateGraph(ResolveState resolveState, boolean denyDynamicSelectors, boolean denyChangingModules) {
-        AttributesSchemaInternal consumerSchema = resolveState.getAttributesSchema();
+        ImmutableAttributesSchema consumerSchema = resolveState.getConsumerSchema();
         for (ModuleResolveState module : resolveState.getModules()) {
             ComponentState selected = module.getSelected();
             if (selected != null) {
-                ResolutionFailureHandler resolutionFailureHandler = resolveState.getVariantSelector().getFailureProcessor();
+                ResolutionFailureHandler resolutionFailureHandler = resolveState.getVariantSelector().getFailureHandler();
                 if (selected.isRejected()) {
                     GradleException error = new GradleException(selected.getRejectedErrorMessage());
                     attachFailureToEdges(error, module.getIncomingEdges());
                     // We need to attach failures on unattached dependencies too, in case a node wasn't selected
                     // at all, but we still want to see an error message for it.
-                    attachFailureToEdges(error, module.getUnattachedDependencies());
+                    attachFailureToEdges(error, module.getUnattachedEdges());
                 } else {
                     if (module.isVirtualPlatform()) {
                         attachMultipleForceOnPlatformFailureToEdges(module);
                     } else if (selected.hasMoreThanOneSelectedNodeUsingVariantAwareResolution()) {
-                        validateMultipleNodeSelection(consumerSchema, module, selected, resolutionFailureHandler);
+                        validateMultipleNodeSelection(consumerSchema, module, selected, resolutionFailureHandler, resolveState.getAttributeSchemaServices());
                     }
                     if (denyDynamicSelectors) {
                         validateDynamicSelectors(selected);
@@ -398,9 +416,9 @@ public class DependencyGraphBuilder {
                 attachMultipleForceOnPlatformFailureToEdges(module);
             }
         }
-        List<EdgeState> incomingRootEdges = resolveState.getRoot().getIncomingEdges();
-        if (!incomingRootEdges.isEmpty()) {
-            String rootNodeName = resolveState.getRoot().getResolvedConfigurationId().getConfiguration();
+
+        if (resolveState.getRoot().wasIncomingEdgeAdded()) {
+            String rootNodeName = resolveState.getRoot().getMetadata().getName();
             DeprecationLogger.deprecate(
                     String.format(
                         "While resolving configuration '%s', it was also selected as a variant. Configurations should not act as both a resolution root and a variant simultaneously. " +
@@ -491,54 +509,45 @@ public class DependencyGraphBuilder {
      * Validates that all selected nodes of a single component have compatible attributes,
      * when using variant aware resolution.
      */
-    private static void validateMultipleNodeSelection(AttributesSchemaInternal consumerSchema, ModuleResolveState module, ComponentState selected, ResolutionFailureHandler resolutionFailureHandler) {
+    private static void validateMultipleNodeSelection(
+        ImmutableAttributesSchema consumerSchema,
+        ModuleResolveState module,
+        ComponentState selected,
+        ResolutionFailureHandler resolutionFailureHandler,
+        AttributeSchemaServices attributeSchemaServices
+    ) {
         Set<NodeState> selectedNodes = selected.getNodes().stream()
             .filter(n -> n.isSelected() && !n.isAttachedToVirtualPlatform() && !n.hasShadowedCapability())
             .collect(Collectors.toSet());
+
         if (selectedNodes.size() < 2) {
             return;
         }
+
         Set<Set<NodeState>> combinations = Sets.combinations(selectedNodes, 2);
         Set<NodeState> incompatibleNodes = new HashSet<>();
+
+        AttributeMatcher matcher = attributeSchemaServices.getMatcher(consumerSchema, selected.getMetadata().getAttributesSchema());
         for (Set<NodeState> combination : combinations) {
             Iterator<NodeState> it = combination.iterator();
             NodeState first = it.next();
             NodeState second = it.next();
-            assertCompatibleAttributes(first, second, incompatibleNodes, consumerSchema);
-        }
-        if (!incompatibleNodes.isEmpty()) {
-            AbstractResolutionFailureException variantsSelectionException = resolutionFailureHandler.incompatibleArtifactVariantsFailure(consumerSchema, selected, incompatibleNodes);
-            for (EdgeState edge : module.getIncomingEdges()) {
-                edge.failWith(variantsSelectionException);
-            }
-        }
-    }
 
-    private static void assertCompatibleAttributes(NodeState first, NodeState second, Set<NodeState> incompatibleNodes, AttributesSchemaInternal consumerSchema) {
-        ImmutableAttributes firstAttributes = first.getMetadata().getAttributes();
-        ImmutableAttributes secondAttributes = second.getMetadata().getAttributes();
-        ImmutableSet<Attribute<?>> firstKeys = firstAttributes.keySet();
-        ImmutableSet<Attribute<?>> secondKeys = secondAttributes.keySet();
-        for (Attribute<?> attribute : Sets.intersection(firstKeys, secondKeys)) {
-            CompatibilityRule<Object> rule = consumerSchema.compatibilityRules(attribute);
-            Object v1 = firstAttributes.getAttribute(attribute);
-            Object v2 = secondAttributes.getAttribute(attribute);
-            // for all commons attributes, make sure they are compatible with each other
-            if (!compatible(rule, v1, v2) && !compatible(rule, v2, v1)) {
+            if (!matcher.areMutuallyCompatible(first.getMetadata().getAttributes(), second.getMetadata().getAttributes())) {
                 incompatibleNodes.add(first);
                 incompatibleNodes.add(second);
             }
         }
-    }
 
-    private static boolean compatible(CompatibilityRule<Object> rule, @Nullable Object v1, @Nullable Object v2) {
-        if (Objects.equals(v1, v2)) {
-            // Equal values are compatible
-            return true;
+        if (!incompatibleNodes.isEmpty()) {
+            Set<VariantGraphResolveMetadata> incompatibleNodeMetadatas = incompatibleNodes.stream()
+                .map(NodeState::getMetadata)
+                .collect(Collectors.toSet());
+            AbstractResolutionFailureException variantsSelectionException = resolutionFailureHandler.incompatibleMultipleNodesValidationFailure(matcher, selected.getMetadata(), incompatibleNodeMetadatas);
+            for (EdgeState edge : module.getIncomingEdges()) {
+                edge.failWith(variantsSelectionException);
+            }
         }
-        DefaultCompatibilityCheckResult<Object> result = new DefaultCompatibilityCheckResult<>(v1, v2);
-        rule.execute(result);
-        return result.hasResult() && result.isCompatible();
     }
 
     private static void attachMultipleForceOnPlatformFailureToEdges(ModuleResolveState module) {
